package org.myhowto.j2me.samples;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.io.*;
import java.io.*;

public class ProxyHTTPMIDlet extends MIDlet implements CommandListener {
	private Display mDisplay;
	private Form mForm;
	private TextField mURL;
	private Thread mHttpThread;
	private StringItem mStatusItem;
	private Command mURLHitCommand;
  
  	public void startApp() {
  		mForm = new Form("HTTP Proxy Test");
		
		mURL = new TextField("URL: http://", "", 64, TextField.URL);
		mForm.append(mURL);
		
		mStatusItem = new StringItem("Status: ", "READY");
		mForm.append(mStatusItem);
		
      		mForm.addCommand(new Command("Exit", Command.EXIT, 2));
		mURLHitCommand = new Command("Hit URL", Command.OK, 1);
      		mForm.addCommand(mURLHitCommand);
      		mForm.setCommandListener(this);
      		mDisplay = Display.getDisplay(this);
    		mDisplay.setCurrent(mForm);
	}
  
  	public void pauseApp() {
  	}
  
  	public void destroyApp(boolean unconditional) {
  	}
  
  	public void commandAction(Command c, Displayable s) {
    		if (c.getCommandType() == Command.EXIT) {
      			notifyDestroyed();
		} else if (c == mURLHitCommand) {
			if (mHttpThread == null) {
				mForm.removeCommand(mURLHitCommand);
				mHttpThread = new Thread(new ProxyHttpThread());
				mHttpThread.start();
			}
		}
	}

	private class ProxyHttpThread implements Runnable {
		public void run() {
			String proxyHost = getAppProperty("HTTP-Proxy-Host");
			String proxyPort = getAppProperty("HTTP-Proxy-Port");

			SocketConnection conn = null;
			OutputStream os = null;
			InputStream is = null;
			mStatusItem.setText("Connecting to the proxy...");
			try {
				// opening the socket connection
				conn = (SocketConnection)Connector.open(
						"socket://" + proxyHost + ":" + proxyPort);
				conn.setSocketOption(SocketConnection.DELAY, 1);
				conn.setSocketOption(SocketConnection.LINGER, 0);
				conn.setSocketOption(SocketConnection.RCVBUF, 8192);
				conn.setSocketOption(SocketConnection.SNDBUF, 512);
				os = conn.openOutputStream();
				is = conn.openInputStream();

				// connection is ready
				mStatusItem.setText("Connected, sending request header...");
				os.write(getRequestHeader().getBytes());
				os.flush();

				// now reading the response
				mStatusItem.setText("Request header sent, waiting for response...");
				byte[] buf = new byte[1024];
				int contentLength = -1;
				int responseStatus  = 0;
				ByteArrayOutputStream headerStream = new ByteArrayOutputStream(800);
				byte numCRs = 0;
				byte numLFs = 0;
				int bytesToGo = 1;
				while(bytesToGo > 0) {
					int count = is.read(buf);
					if (count == -1) {
						// end-of-stream
						break;
					}
					if (headerStream != null) {
						for(int i = 0; i < count; i++) {
							if(buf[i] == '\n') {
								numLFs++;
								if (numLFs == 2 && numCRs == 2) {
									// end of HTTP 1.1 header (ascii)
									String hdrText = new String(headerStream.toByteArray());
									responseStatus = Integer.parseInt(hdrText.substring(9, 12));
									int clPos = hdrText.toLowerCase().indexOf("content-length: ");
									if (clPos == -1) {
										// reading till the end of stream
										bytesToGo = Integer.MAX_VALUE;
									} else {
										contentLength = Integer.parseInt(
												hdrText.substring(clPos + 16, 
													hdrText.indexOf("\r\n", clPos + 17)));
										bytesToGo = contentLength - (count - i - 1);
									}

									mStatusItem.setText("Got header, receiving body...");
									headerStream = null;
									break;
								}
							} else if (buf[i] == '\r') {
								numCRs++;
							} else {
								numLFs = 0;
								numCRs = 0;
							}
							headerStream.write(buf[i]);
						}
					} else {
						bytesToGo -= count;
						if (contentLength != -1) {
							mStatusItem.setText("Loading body, " + bytesToGo + 
								       " more byte(s) to receive");
						}
					}
				}

				// Got everything
				mStatusItem.setText("HTTP status=" + responseStatus + ", content-length: " +
					       (contentLength == -1 ? "unspecified" : String.valueOf(contentLength)));
			} catch(IOException e) {
				e.printStackTrace();
				mStatusItem.setText("I/O error: " + e.getMessage());
			} finally {
				try {
					if (is != null) {
						is.close();
					}
					if (os != null) {
						os.close();
					}
					if (conn != null) {
						conn.close();
					}
				} catch (IOException e) {
					// c'est la vie...
				}
				mForm.addCommand(mURLHitCommand);
				mHttpThread = null;
			}
		}

		private String getRequestHeader() {
			StringBuffer buf = new StringBuffer(512);
			String targetURL = mURL.getString();
			buf.append("GET http://" + targetURL + " HTTP/1.1\n");
			// extracting the target host and port number
			int portDelPos = targetURL.indexOf(':');
			int firstSlashPos = targetURL.indexOf('/');
			String targetHost = targetURL;
			String targetPort = "80";
			if (portDelPos != -1 && portDelPos < firstSlashPos) {
				// has port specified
				targetHost = targetURL.substring(0, portDelPos);
				if (firstSlashPos == -1) {
					targetPort = targetURL.substring(portDelPos + 1);
				} else {
					targetPort = targetURL.substring(portDelPos + 1, firstSlashPos);
				}
			} else {
				// no port - using default
				if (firstSlashPos != -1) {
					targetHost = targetURL.substring(0, firstSlashPos);
				}
			}
			buf.append("Host: ");
			buf.append(targetHost);
			buf.append(':');
			buf.append(targetPort);
			buf.append('\n');
			buf.append("Accept: */*\n");
			buf.append("Cache-control: no-transform\n");
			// if you want keep-alive - remove the following line
			buf.append("Connection: close\n");
			buf.append("User-Agent: MIDP2.0\n");
			buf.append('\n');

			return buf.toString();
		}
	}
}
